% This Matlab file contains analysis code used in:
%
% Ezzyat et al. (2017)  Direct brain stimulation modulates encoding states
%   and memory performance in humans. Current Biology, 27(9), 1251-1258.
%
% Feel free to use this code for your own data. Contact Youssef Ezzyat
% (yezzyat@sas.upenn.edu) with comments, questions, or bug reports.
% Unfortunately I cannot support adapting this code for other projects.
%
% Some of the code used here relies on other code that is part of the Penn
% Computational Memory Lab's (CML) EEG Toolbox, iEEGDataBase, and
% Behavioral Toolbox. For information see http://memory.psych.upenn.edu/Software.
%
% Some of the classification code is adapted from the Princeton MVPA toolbox
% (http://code.google.com/p/princeton-mvpa-toolbox/), and relies on the 
% Matlab implementation of the LIBLINEAR package:
%   R.-E. Fan, K.-W. Chang, C.-J. Hsieh, X.-R. Wang, and C.-J. Lin.
%   LIBLINEAR: A Library for Large Linear Classification, Journal of
%   Machine Learning Research 9(2008), 1871-1874.
%   Software available at http://www.csie.ntu.edu.tw/~cjlin/liblinear


%% Spectral power
% See multiphasevec2.m      Penn CML EEG toolbox function
%   Performs spectral decomposition using wavelets.


%% Classification
% See train_L2_liblinear.m in EzzyEtal17_CodeArchive
%   L2-penalized LR model cross-validated by session. Uses the Liblinear
%   2.11 matlab implementation:

%       R.-E. Fan, K.-W. Chang, C.-J. Hsieh, X.-R. Wang, and C.-J. Lin.
%       LIBLINEAR: A library for large linear classification Journal of
%       Machine Learning Research 9(2008), 1871-1874.


%% Forward model weights
% Given an N observations X M features spectral power matrix (pattern),
% an array of N classifier output probabilities (classout), and an array
% of classifier weights (classwgts) compute the forward model using the
% method from:
%   Haufe S et al (2014). On the interpretation of weight vectors of linear
%	models in multivariate neuroimaging. NeuroImage, 87, 96-110.

s       = logit(classout);
covx    = cov(pattern);
covs    = cov(s);
W       = classwgts(1:end-1); % Exclude constant term
A       = covx * W' / covs;


%% Subsequent memory effect
% See compute_sme.m in EzzyEtal17_CodeArchive
%   Calculates subsequent memory effect. Makes use of Penn CML EEG toolbox
%   functions.


%% Stim-NoStim delta memory
% Given a subject code e.g. 'R1001P'
events = get_sub_events('RAM_FR1',subject);
events(~strcmp({events.type},'WORD')) = [];
prec   = [nanmean([events([events.stimList]==1).recalled]),...
    nanmean([events([events.stimList]==0).recalled]),...
    nanmean([events.recalled]==1)];

deltamem = 100*((prec(1)-prec(2))./prec(3));


%% Serial position and conditional response probability curves
% This section makes use of Penn CML Behavioral Toolbox functions spc.m,
% crp.m, and FRdata.m, and the EEG Toolbox function get_sub_events.m

% Given a subject code e.g. 'R1001P'
task = 'RAM_FR1'; subject = 'R1001P';
events = get_sub_events(task,subject);
d = FRdata(events,'list');
SerialPosCurve = spc(d.recalls,d.subject,d.listLength);
CondRespProb = crp(d.recalls,d.subject,d.listLength);


%% Recall on PostStim items and matched items from NoStim lists
% Apply classifier model to spectral power data, to derive pre-stimulation
% classifier estimates. Uses liblinear predict function.
%
%   events      = behavioral events structure
%   fullmodel   = classifier oubject from do_binary_cross_validation_loso.m
%   pattern     = pattern of spectral power features

y = [events.recalled]';
y(y==0) = -1;
[~, ~, acts] = predict(y,sparse(double(pattern)), fullmodel.model,'-b 1');
acts = acts(:,1);

% Compute PostStim/NoStim recall as a function of brain state below
% classifier threshold.
thresh = 0.50;
low = acts < thresh;
itemlag = 2;
poststim_prec   = nanmean([events(find([events.isStim]==1 &...
    ismember([events.serialpos],1:2:9) & low)+itemlag).recalled]);
postnostim_prec = nanmean([events(find(isnan([events.stimAnode]) & ...
    ismember([events.serialpos],1:2:9) & low)+itemlag).recalled]);


%% Extracting ROIs from an SME map for computing the spectral tilt
% Load output grpdata from compute_sme.m

% Find significant clusters, corrected using fdr.m (Penn CML EEG Toolbox),
% for use in extracting stimulation-evoked power.
[~,p,~,stat] = ttest(grpdata.ts,0,'dim',3);
fdrp = fdr(p(:),0.05);
sme_mask = p < fdrp;
CC = bwconncomp(sme_mask);

% Load in an array (frequencies X ROIs X subjects) of t-stats comparing
% PostStim to PostNostim conditions.

% Grab the PostStim-PostNoStim data using the SME mask
for isubj = 1:size(postdata.t,3)
    for iclust = 1:CC.NumObjects
        tmpdat = postdata.t(:,:,isubj);
        clustdata.ts(isubj,iclust) = nanmean(tmpdat(CC.PixelIdxList{iclust}));
    end
end


