function [grpdata] = compute_sme(SubjList)
% function [grpdata] = compute_sme(SubjList)
%
% Computes the subsequent memory effect (SME) given a set of spectral power
% patterns, for a set of subjects. Returns a Frequency X ROI X Subject
% array of spectral power values for recalled/not recalled events, and of
% t-stats for the comparison between the two.

%% Set parameters
nfreqs      = nan;      % number of wavelet frequencies
timebins    = nan;      % word encoding period
powdir      = '';       % path to spectral power patterns

% Get ROI list
inds    = surface_DK_ROI(getBipolarSubjElecs(SubjList{1}),SubjList{1});
rois    = fieldnames(inds.ROI); rois(strcmp(rois,'remain')) = [];

%% Compute the SME
grpdata.rec = nan(nfreqs,length(rois),length(SubjList));
grpdata.nrec=grpdata.rec;
grpdata.ts  = grpdata.rec;
grpdata.ps  = grpdata.rec;

for isubj = 1:length(SubjList)
        
        subject     = SubjList{isubj};
        
        % The next two lines use Penn Computational Memory Lab EEG toolbox 
        % functions for loading electrode localization and behavioral 
        % events files.
        bp          = getBipolarSubjElecs(subject,1,1,1);
        events      = get_sub_events('RAM_FR1',subject);
        
        
        sesslist    = unique([events.session]);
        inds        = surface_DK_ROI(bp,subject);
        
        for iroi = 1:length(rois)
            roielecs        = bp(logical(inds.ROI.(rois{iroi})));
            elecdata.rec    = nan(nfreqs,length(roielecs));
            elecdata.nrec   = elecdata.rec;
            elecdata.ts     = elecdata.rec;
            elecdata.ps     = elecdata.rec;
            
            for ielec = 1:length(roielecs)
                pow = []; wordev = [];
                
                for isess = 1:length(sesslist)
                    cd(fullfile(powdir,subject,num2str(sesslist(isess))));
                    sessev  = events([events.session]==sesslist(isess));
                    words   = strcmp({sessev.type},'WORD');
                    wordev  = [wordev sessev(words)];
                    
                    % Loads spectral power data saved as one .mat file per
                    % electrode, in a variable named sessOutput.pow.
                    % Dimensions = P frequencies x Q timebins x R events
                    load(bp(ielec).eNames);
                    tmp     = sessOutput.pow(:,timebins,words);
                    m       = nanmean(nanmean(tmp,2),3);
                    sd      = nanstd(nanmean(tmp,2),[],3);
                    tmp     = zScorePow(tmp,m,sd);op
                    pow     = cat(3,pow,tmp);
                end
                
                elecdata.rec(:,ielec)     = nanmean(nanmean(pow(:,:,[wordev.recalled]==1),3),2);
                elecdata.nrec(:,ielec)    = nanmean(nanmean(pow(:,:,[wordev.recalled]==0),3),2);
                [~,p,~,stat]              = ttest2(squeeze(nanmean(pow(:,:,[wordev.recalled]==1),2)),...
                    squeeze(nanmean(pow(:,:,[wordev.recalled]==0),2)),'dim',2);
                elecdata.ts(:,ielec)      = stat.tstat;
                elecdata.ps(:,ielec)        = p;
            end
            
            grpdata.rec(:,iroi,isubj)       = nanmean(elecdata.rec,2);
            grpdata.nrec(:,iroi,isubj)      = nanmean(elecdata.nrec,2);
            grpdata.ts(:,iroi,isubj)        = nanmean(elecdata.ts,2);
            grpdata.ps(:,iroi,isubj)        = p;
        end

end
        

function [zPowMat] = zScorePow(PowMat,BasePowMean,BasePowSTD)

zPowMat = (PowMat - repmat(BasePowMean,[1 size(PowMat,2), size(PowMat,3)]))...
     ./repmat(BasePowSTD,[1 size(PowMat,2), size(PowMat,3)]);
 