function scratchpad = train_L2_liblinear(train_pats,train_targets,train_session,...
    train_list,train_timepoint,C,xvaltype,stdze)

% Performs L2-penalized logistic regression classification. This function
% was adapted from the Princeton MVPA toolbox (http://code.google.com/p/princeton-mvpa-toolbox/)

% Classification is performed using the Matlab implementation of LIBLINEAR:
%
%   R.-E. Fan, K.-W. Chang, C.-J. Hsieh, X.-R. Wang, and C.-J. Lin. 
%   LIBLINEAR: A Library for Large Linear Classification, Journal of 
%   Machine Learning Research 9(2008), 1871-1874.
%   Software available at http://www.csie.ntu.edu.tw/~cjlin/liblinear


if ~exist('train_timepoint','var')
    train_timepoint = ones(1,size(train_pats,1));
end

    

if ~issparse(train_pats);
    if ~isa(train_pats,'double')
        train_pats = double(train_pats);
    end
    train_pats = sparse(train_pats');
end

if ~isvector(train_targets)
    tmp = train_targets;
    train_targets = [];
    
    for k = 1:size(tmp,1)
        train_targets(tmp(k,:)==1)=k;
    end
    train_targets = train_targets';
end

train_targets(train_targets==1)=1;
train_targets(train_targets==2)=-1;

% parameter values for C here
if ~exist('C','var')
    C = logspace(log10(1e-6),log10(1e4),22);
end
T = unique(train_timepoint);

if length(unique(train_session))==1
    xvaltype = 'list';
end

if ~exist('xvaltype','var')
    xvaltype = 'session';
end

switch xvaltype
    case 'list'
        cnt = 1;
        for k = 1:length(C)
            for t = 1:length(T)
                liblin_param = ['-q -c ' sprintf('%f',C(k)) ' -s 0 -B 1'];
                
                [ret,decvals] = do_binary_cross_validation_lolo(train_targets,...
                    train_pats(:,train_timepoint==T(t)), liblin_param, train_session, train_list,stdze);
                
                C_all(cnt) = C(k);
                T_all(cnt) = T(t);
                AUC_all(cnt) = ret;
                decvals_all{cnt} = decvals;
                
                cnt=cnt+1;
            end
        end
        
    case 'session'
        cnt = 1;
        for k = 1:length(C)
            for t = 1:length(T)
                liblin_param = ['-q -c ' sprintf('%f',C(k)) ' -s 0 -B 1'];
                [ret,decvals] = do_binary_cross_validation_loso(train_targets,...
                    train_pats(:,train_timepoint==T(t)), liblin_param, train_session, train_list,stdze);
                
                C_all(cnt) = C(k);
                T_all(cnt) = T(t);
                AUC_all(cnt) = ret;
                decvals_all{cnt} = decvals;
                
                cnt=cnt+1;
            end
        end
        
end

% update to work with multiple timpoints
scratchpad.hyper.C = C_all(find(AUC_all==max(AUC_all),1,'first'));
scratchpad.hyper.AUC = AUC_all(find(AUC_all==max(AUC_all),1,'first'));
scratchpad.hyper.T = T_all(find(AUC_all==max(AUC_all),1,'first'));

scratchpad.hyper.C_all = C_all;
scratchpad.hyper.AUC_all = AUC_all;
scratchpad.hyper.T_all = T_all;
scratchpad.hyper.decvals_all = decvals_all;

n_pos = numel(find(train_targets==1))/numel(train_targets);
n_neg = numel(find(train_targets==-1))/numel(train_targets);
mean_tmp = mean([n_pos n_neg]);
n_pos = n_pos/mean_tmp;n_neg = n_neg/mean_tmp;
% invert the weights on the penalty, to account for class imbalance
tmp_pos = n_pos;
n_pos = n_neg;
n_neg = tmp_pos; clear tmp_pos;

% formatting
C = sprintf('%f',scratchpad.hyper.C);
n_pos = sprintf('%f',n_pos);
n_neg = sprintf('%f',n_neg);


param = ['-q -c ' C ' -s 0 -B 1'];
%param = ['-q -c ' C ' -s 2 -B 1'];
param = [param ' -w1 ' n_pos ' -w-1 ' n_neg];

% standardize the training features, and store mean and sd to normalize
% the test features - need to have this operate per session
if stdze
    [tmp, scratch] = standardize(train_pats(:,train_timepoint==scratchpad.hyper.T), train_session);
else
    scratch.mean    = [];
    scratch.std     = [];
    scratch.sess    = unique(train_session);
    tmp = train_pats(:,train_timepoint==scratchpad.hyper.T);
end

scratchpad.mean = scratch.mean;
scratchpad.std = scratch.std;
scratchpad.sess = scratch.sess;

model = train(train_targets,tmp,param);

scratchpad.model = model;

function [pat, scratch] = standardize(pat, session)

sess = unique(session);

scratch = struct();
scratch.sess = sess;

for cS = 1:length(sess)
    
scratch.mean(cS,:) = nanmean(pat(session==sess(cS),:));
scratch.std(cS,:) = nanstd(pat(session==sess(cS),:));

pat(session==sess(cS),:) = bsxfun(@minus, pat(session==sess(cS),:),...
    scratch.mean(cS,:));
pat(session==sess(cS),:) = bsxfun(@rdivide, pat(session==sess(cS),:),...
    scratch.std(cS,:));

end
